# coding: utf-8
from __future__ import unicode_literals, absolute_import
import datetime
import json


from kinder.core.children.tests import factory_child
from kinder.core.declaration.tests import factory_declaration
from kinder.core.declaration_status.models import DeclarationStatus
from kinder.core.unit.models import Unit, UnitKind
# from kinder.core.unit.helpers import get_dou_for_mo
from kinder.core.unit.tests import factory_unit

from gisdo.models import ReportFormRow, ReportForm
from gisdo.collector import DataCollector
from gisdo.service import ReportService
from gisdo.utils import AverageWaitingTimeWrapperEncoder
from gisdo.utils import UnitHelper
from .factory_gisdo import GisdoUnitFactory


def get_dou_for_mo(mo):
    """
    Возвращает список дочерних ДОУ
    :param mo:
    :return:
    """
    if mo.kind_id != UnitKind.MO:
        return []
    return list(mo.get_descendants().filter(kind__id=UnitKind.DOU))


class TestReportService(ReportService):
    """Запускает сборку отчета ФО в синхронном режиме,
    не рекомендуется для использования для реальных данных,
    только для тестовых, для небольшого объема,
    для проверки работоспособности тестов.
    """

    def _collect_data(self, mo_id, report_form_id):
        """копипаст gisdo.service.collect_data"""
        mo = Unit.objects.get(id=mo_id)
        report_form = ReportForm.objects.get(id=report_form_id)

        main_report_unit = report_form.unit
        unit_helper = UnitHelper(main_report_unit)

        data = None
        exception = None

        try:
            data = DataCollector(mo, unit_helper).collect(report_form)
        # Отложенное исключение
        except Exception as e:
            exception = e
        return mo.id, data, exception

    def collect(self, send=True):
        self._report_form.save()
        collect_data_results = (
            self._collect_data(
                mo.id, self._report_form.id
            ) for mo in self._unit_helper.get_report_units())

        # Обрабатываем результаты завершившихся задач
        for (mo_id, task_result, exception) in collect_data_results:
            # Выбрасываем отложенное исключение
            if exception:
                raise exception
            self._update_data(task_result)

        if self._unit_helper.is_region:
            ReportFormRow.objects.get_or_create(
                report=self._report_form,
                unit=self._unit_helper.get_main_unit(),
                applications=json.dumps(self._cache[self.APPLICATIONS]),
                queues=json.dumps(
                    self._cache[self.QUEUES],
                    cls=AverageWaitingTimeWrapperEncoder),
                enrolled=json.dumps(self._cache[self.ENROLLED]),
                capacities=json.dumps(self._cache[self.CAPACITIES]))

        self._report_form.in_progress = False
        self._report_form.date = datetime.datetime.now()
        self._report_form.save()


def make_dou_list(count_mo=3, count_dou=10):
    """Создает учреждения и ссылку в гисдо модели на них
    :param count_mo:
    :param count_dou:
    :return:
    """
    region = factory_unit.UnitRFactory()
    mo_list = []
    dou_list = []
    for i in range(0, count_mo):
        mo = factory_unit.UnitMoFactory(parent=region)
        mo_list.append(mo)
        for j in range(0, count_dou):
            dou = factory_unit.UnitDouFactory(parent=mo)
            GisdoUnitFactory(unit=dou)
            dou_list.append(dou)
    return region, mo_list, dou_list


def make_decl_list(mo_list, statuses, count_decl=10):
    """Создает рандомные заявления
    :param mo_list:
    :param statuses:
    :param count_decl:
    :return:
    """
    decl_list = []
    for mo in mo_list:
        dou_list = get_dou_for_mo(mo)
        for dou in dou_list:
            for status in statuses:
                for i in range(0, count_decl):
                    child = factory_child.ChildF.create()
                    decl_list.append(factory_declaration.DeclarationF.create(
                        children=child, mo=mo,
                        status=DeclarationStatus.get_by_code(status))
                    )